/* 
 * Copyright (C) 1997, 1998 Defence Research Establishment, Dept. of Comm.
 * Systems, FOA 72, Sweden.
 * Currently maintained by Christian Jnsson FOA 72 <chj@lin.foa.se>.
 *
 * This file is part of ETFTP version 1.1.3.
 *
 *   ETFTP is free software; you can redistribute it and/or modify it
 *   under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   ETFTP is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with ETFTP; see the file GPL.  If not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *   Boston, MA 02111-1307, USA.
 *
 * Code:
 *
 *----------------------------------------------------------------------
 * Filenamne:   cmdsubr.c
 * Class:       CmdSubr
 * Created:     0.1, 970430, Per-Erik Forssn
 * Revised:     1.1, 980219, Torbjrn Frostemark
 * 
 * Description: Miscellaneous methods for user command processing.
 * Remarks:     -
 *
 * Private Metods:      -
 * Private Attributes:  -
 *----------------------------------------------------------------------
 */

#include    "clu.h"
#include    "cmdsubr.h"
#include    "cmd.h"

Cmds    commands[] = {  /* keep in alphabetical order for binary search */
  {"?",            CmdHelp,
   "?                                      Display this text."}
   ,
  {"ascii",        CmdASCII,
   "ascii                                  Select ascii transfer mode."}
   ,
  {"autoadapt",    CmdAutoadapt,
   "autoadapt on|off                       Turn autoadapt function on or off."}
   ,
  {"baudrate",     CmdBaudrate,
   "baudrate <rate>                        Set estimated data-transfer-rate"}
   ,
  {"binary",       CmdBinary,
   "binary                                 Select binary transfer mode."}
   ,
  {"blocksize",    CmdBlocksize,
   "blocksize <size>                       Set size of data blocks."}
   ,
  {"bufferblock",  CmdBufferblock,
   "bufferblock <no>                       Set number of blocks per buffer."}
   ,
  {"burstsize",    CmdBurstsize,
   "burstsize <size>                       Set number of blocks per burst."}
   ,
  {"connect",      CmdConnect,
   "connect <hostname> [<port>]            Set address of remote host."}
   ,
  {"exit",         CmdExit,
   "exit                                   End etftp."}
   ,
  {"get",          CmdGet,
   "get [<host>:]<remotefile> <localfile>  Receive remote file."}
   ,
  {"hash",         CmdHash,
   "hash on|off                            Set hash function."}
   ,
  {"help",         CmdHelp,
   "help                                   Display this text."}
   ,
  {"localport",    CmdLocalport,
   "localport <port>                       Set receive port number for client."}
   ,
  {"mode",         CmdMode,
   "mode <mode>                            Set binary or ascii transfer mode."}
   ,
  {"multibuff",    CmdMultibuff,
   "multibuff <no>                         Set number of buffers {1,2}"}
   ,
  {"put",          CmdPut,
   "put <localfile> [<host>:]<remotefile>  Send a file to remote host."}
   ,
  {"quit",         CmdExit,
   "quit                                   End etftp."}
   ,
  {"radiodelay",   CmdRadiodelay,
   "radiodelay <seconds>                   Set the maximum turnover-time."}
   ,
  {"status",       CmdStatus,
   "status                                 Display current settings."}
   ,
  {"trace",        CmdTrace,
   "trace on|off                           Set trace (log) function."}
   ,
  {"verbose",      CmdVerbose,
   "verbose on|off                         Set verbose function."}
};

#define NCMDS   (sizeof(commands) / sizeof(Cmds))

int		nCmds = NCMDS;
static char     line[MAXLINE] = { 0 };
static char     *lineptr = NULL;

/*----------------------------------------------------------------------
 * CSGetLine
 *
 *      Fetch a new command line.
 *
 * Results:
 *      Return 1 on success or 0 on failure.
 *
 * Side effects:
 *      Puts line in variable lineptr.
 *----------------------------------------------------------------------
 */

int CSGetLine(fp)
     FILE *fp;
{
  if (fgets(line, MAXLINE, fp) == NULL) {
    return 0;               /* error or end-of-file */
  }
  lineptr = line;
  
  return 1;
}

/*----------------------------------------------------------------------
 * CSGetToken
 *
 *      Fetch the next token from the input stream.
 *      We use the line that was set up in the most previous call to
 *      CSGetLine().
 *
 * Results:
 *      Return a pointer to the token (the argument), or NULL if no more exist.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

char *CSGetToken(token)
     char token[];
{
  register int    c;
  register char   *tokenptr;

  /* skip leading white space */
  while ((c = *lineptr++) == ' ' || c == '\t') {}
  
  if (c == '\0' || c == '\n') {
    return NULL;		/* nothing there */
  }

  tokenptr = token;
  *tokenptr++ = c;		/* first char of token */
  
  /*
   *  Now collect everything up to the next space, tab, newline, or null.
   */
  
  while ((c = *lineptr++) != ' ' && c != '\t' && c != '\n' && c != '\0') {
    *tokenptr++ = c;
  }
  *tokenptr = 0;		/* null terminate token */

  return token;
}

/*----------------------------------------------------------------------
 * CSCheckEnd
 *
 *      Verify that there aren't any more tokens left on a command line.
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      Reports trailing tokens.
 *----------------------------------------------------------------------
 */

void CSCheckEnd()
{
  if (CSGetToken(tempToken) != NULL) {
    DEBUG0("CSCheckEnd: Trailing garbage");
  }
}

/*----------------------------------------------------------------------
 * CSDoCmd
 *
 *      Execute a command.
 *      Call the appropriate function.  If all goes well, that function will
 *      return, otherwise that function may call an error handler, which will
 *      call longjmp() and branch back to the main command processing loop.
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

void CSDoCmd(cmdPtr)
     char    *cmdPtr;
{
  register int    i;

  if ( (i = CSBinary(cmdPtr, nCmds)) < 0) {
    CSErrCmd(cmdPtr);
  }
  
  (*commands[i].cmdFunc)();
  
  CSCheckEnd();
}

/*----------------------------------------------------------------------
 * CSBinary
 *
 *      Perform a binary search of the command table
 *      to see if a given token is a command.
 *
 * Results:
 *      Index of command in list or -1 if not found.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */
int CSBinary(word, n)
    char    *word;
    int     n;
{
  register int    low, high, mid, cond;

  low  = 0;
  high = n - 1;
  while (low <= high) {
    mid = (low + high) / 2;
    if ( (cond = strcmp(word, commands[mid].cmdName)) < 0) {
      high = mid - 1;
    } else if (cond > 0) {
      low = mid + 1;
    } else {
      return mid;		/* found it, return index in array */
    }
  }
  return -1;			/* not found */
}

/*----------------------------------------------------------------------
 * CSStripHost
 *
 *      Take a "host:file" character string and separate the "host"
 *      portion from the "file" portion.
 *
 * Results:
 *
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

void CSStripHost(fname, hname)
     char    *fname;		/* input:  "host:file" or just "file" */
     char    *hname;		/* store "host" name here, if present */
{
  register char   *ptr1, *ptr2;
  
  if ( (ptr1 = strchr(fname, ':')) == NULL) {
    return;			/* there is not a "host:" present */
  }
  
  /*
   * Copy the entire "host:file" into the hname array,
   * then replace the colon with a null byte.
   */

  strcpy(hname, fname);
  ptr2 = strchr(hname, ':');
  *ptr2 = 0;			/* null terminates the "host" string */
  
  /*
   * Now move the "file" string left in the fname array,
   * removing the "host:" portion.
   */

  strcpy(fname, ptr1 + 1);	/* ptr1 + 1 to skip over the ':' */
}

/*----------------------------------------------------------------------
 * CSErrCmd
 *
 *      User command error.
 *      Print out the command line too, for information.
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      Reports error to stderr.
 *----------------------------------------------------------------------
 */

void CSErrCmd(str)
     char    *str;		/* may be a 0-length string, i.e., "" */
{
  fprintf(stderr, "%s: '%s' command error", pName, command);
  if (strlen(str) > 0) {
    fprintf(stderr, ": %s", str);
  }
  fprintf(stderr, "\n");
  fflush(stderr);
  
  longjmp(jmpMainLoop, 1);	/* 1 -> not a timeout, we've already
				 * printed our error message */
}

/*----------------------------------------------------------------------
 * CSOnOff
 *
 *      Get an on or off argument.
 *
 * Results:
 *      -1 other,1 on 0 off.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

int CSOnOff()
{
  if (CSGetToken(tempToken) == NULL) {
    CSErrCmd("On or off should be specified.");
    return -1;
  } else {
    if (strcmp(tempToken, "on") == 0) {
      return 1;
    } else if (strcmp(tempToken, "off") == 0) {
      return 0;
    } else {
      CSErrCmd("On or off should be specified.");
      return -1;
    }
  }
}

/*----------------------------------------------------------------------
 * CSCheckBurstRate
 *
 *      Check valid values for calulation of burstrate.
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

void CSCheckBurstRate(char *str, unsigned short newBlockSize,
		      unsigned short newBurstSize,
		      long newBaudRate)
{
  /*  if (((newBlockSize+72)*newBurstSize*8/newBaudRate*1000) > 65535) {*/
  if (((newBlockSize+72)*newBurstSize*8/newBaudRate) > 65535) {
    if (interactive) {
      /* interactive mode, exit to main loop */
      CSErrCmd("requested value makes burst rate invalid. Aborting.");
    } else {
      /* batch file used, abort program */
      printf("Current %s makes burst rate invalid. Terminating ETFTP.\n", str);
      exit(0);
    }
  }
}







