/* 
 * Copyright (C) 1997, 1998 Defence Research Establishment, Dept. of Comm.
 * Systems, FOA 72, Sweden.
 * Currently maintained by Christian Jnsson FOA 72 <chj@lin.foa.se>.
 *
 * This file is part of ETFTP version 1.1.3.
 *
 *   ETFTP is free software; you can redistribute it and/or modify it
 *   under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   ETFTP is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with ETFTP; see the file GPL.  If not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *   Boston, MA 02111-1307, USA.
 *
 * Code:
 *
 *----------------------------------------------------------------------
 * Filenamne:   tron.c
 * Class:       Tron
 * Created:     0.1, 970420, Per-Erik Forssn
 * Revised:     1.1, 980219, Torbjrn Frostemark
 *
 * Description: Used by the reciever to calculate new transfer-parameters
 *		based on how sucessful the previous buffer transfer was.
 *		Also calculates average throughput.
 * Remarks:     -
 *
 * Private Methods:	-
 *
 * Private Attributes:	static long tRONSentPackets	 No of packets sent
 *			static TRONList *tRONResendList	 List of packets not 
 *					    		 recieved
 *			static int tRONBurstSize;
 *			static int tRONBurstRate;
 *			static int tRONblockSize;
 *			static int tRONBufBlocks;
 *			static int tRONBytesSoFar;
 *			static int tRONStartTime;
 *----------------------------------------------------------------------
 */

#include "tron.h"
#include "etftptime.h"
#include <math.h>
#include <stdlib.h>

/*
 * Attributes to the Tron object.
 */

static long tRONSentPackets = 0;
static TRONList tRONResendList = {NULL, 0};
static int tRONBurstSize = 0;
static int tRONBurstRate = 0;
static int tRONBlockSize = 0;
static int tRONBufBlocks = 0;
static int tRONBytesSoFar = 0;
static int tRONStartTime;

/*----------------------------------------------------------------------
 * TRONStart
 *
 *      Inform TRON of current tranfer parameters.
 *	Called at the start of a file-transfer.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	Starts the timing for transfer speed averageing.
 *----------------------------------------------------------------------
 */

void TRONStart(unsigned short burstSize, unsigned short burstRate, unsigned short blockSize)
{
  tRONBurstSize = burstSize;
  tRONBurstRate = (int) burstRate;
  tRONBlockSize = blockSize;
  tRONBytesSoFar = 0;
  tRONStartTime = TimeGet();	/* For averageing transfer rate */
}

/*----------------------------------------------------------------------
 * TRONReset
 *
 *	Reset packet statistics and the resend-list.
 *	Called when a new buffer is about to be recieved.
 *	bufferSize is number of blocks per buffer.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	None.
 *----------------------------------------------------------------------
 */

void TRONReset(short bufferSize, int fileSize)
{
  short *newMem;
  int	index, blocksLeft;

  /*  printf("TRONReset: Resetting TRON. bufferSize = %d, fileSize = %d\n", bufferSize, fileSize);*/
  free(tRONResendList.list);
  blocksLeft = (fileSize - tRONBytesSoFar) / tRONBlockSize;
  tRONBufBlocks = blocksLeft;	/* If last buffer, there might be fewer blocks */
  if (blocksLeft > bufferSize) {
    tRONBufBlocks = bufferSize; /* Ensure no more than bufferSize blocks */
  }
  newMem =(short *)malloc( tRONBufBlocks * sizeof(short) );
  tRONResendList.size = tRONBufBlocks;
  tRONResendList.list = newMem;
  for (index = 0; index < tRONBufBlocks; index++) {
    tRONResendList.list[index] = (short)index; /* Fill the List */
  }
  tRONSentPackets = tRONBufBlocks; /* Reset packet statistics */

  /*  printf("TRONReset: tRONBytesSoFar = %d, tRONBlockSize = %d\nTRONReset: tRONBufBlocks = %d, blocksLeft = %d\n",
      tRONBytesSoFar, tRONBlockSize, tRONBufBlocks, blocksLeft);*/
}

/*----------------------------------------------------------------------
 * TRONUpdate
 *
 *	Update packet statistics. Called after each recieved DATA and
 *	LDATA-packet.
 *
 * Results:
 *	Returns average transfer rate.
 *
 * Side effects:
 *	Alters resend list and average transfer speed.
 *----------------------------------------------------------------------
 */

double TRONUpdate(short packNum, int packSize)
{
  int index, size;
  int remove = 0;
  short *tl;

  /*printf("TRONUpdate: packNum = %d, packSize = %d\n", packNum, packSize);*/

  size = tRONResendList.size;
  tl = tRONResendList.list;

  /* Check if packet is in list */

  for (index = 0; index < size; index++) {
    if ((tl[index] == packNum) ) {
      remove = 1;
      break;
    }
  }

  /* If packet was there, remove it */

  if (remove) {
    for (index = 0; index < size; index++) {
      if (tl[index] >= packNum) {
	tl[index] = tl[index+1];
      }
    }
    tRONResendList.size--;
    tRONBytesSoFar += packSize;	/* For averageing */
    /*printf("TRONUpdate: tRONBytesSoFar = %d\n", tRONBytesSoFar);*/
  }

  /*tRONBytesSoFar += packSize;*/	/* For averageing */
    
  /* if no time has elapsed return zero */
  if ((TimeGet() - tRONStartTime) == 0) {
    return (double)0;
  }

  return (double)(tRONBytesSoFar / (TimeGet() - tRONStartTime));
}

/*----------------------------------------------------------------------
 * TRONDecide
 *
 *	Calculate new transfer parameters. Called before sending CONTROL_OK.
 *
 * Results:
 *	1 - if changed, 0 - unchanged.
 *
 * Side effects:
 *	None.
 *----------------------------------------------------------------------
 */

int TRONDecide(burstSize,burstRate,blockSize)
    unsigned short *burstSize;
    unsigned short *burstRate;
    unsigned short *blockSize;
{
  /* Check if we should go down in size */
  /* If more than 50% of the packets have to be resent then down.      */
  /* Now tRONSentPackets is the number of packets to be sent (if       */
  /* packets had to be resent this number is bigger than tRONbufBlocks */

  /*  printf("\nTRONDecide: tRONSentPackets = %ld\n", tRONSentPackets);*/

  if (((1.0*(tRONSentPackets - tRONBufBlocks)) / tRONBufBlocks) >= 0.5) {
    /* ---------------- Negotiate down ----------------- */

    if ( TRONHalve(blockSize[0]) >= 16) {
      double tmpBurstRate;

      tRONBlockSize = blockSize[0] = TRONHalve(blockSize[0]);

      tmpBurstRate = ((blockSize[0]+72)*burstSize[0]*8/baudRate);

      if (tmpBurstRate < 0.3) {
	burstRate[0] = 0;
      } else {
	burstRate[0] = (unsigned short) ceil( tmpBurstRate );	/* decrease burstRate */
      }
      return 1; /* blockSize has been halved */
    }

    if ( burstSize[0] > 1) {
      double tmpBurstRate;

      burstSize[0]--;
      tRONBurstSize = burstSize[0];

      if ((tmpBurstRate = ((blockSize[0]+72)*burstSize[0]*8/baudRate)) < 0.3) {
	burstRate[0] = 0;
      } else {
	burstRate[0] = (unsigned short) ceil(tmpBurstRate);	/* decrease burstRate */
      }

      tRONBurstRate = (int) burstRate[0];
      return 1;
    } else { /* last attempt to stabilize thruput */
      burstRate[0] = (unsigned short) tightTimer;      /* Timer fixed by ChJ */
    }

    tRONBurstRate = (int) burstRate[0];

    return 1;
  }

  /*
   * Check if we should go up in size
   * That is if less than 1% have to be resent
   */

  if (((1.0*(tRONSentPackets - tRONBufBlocks)) / tRONBufBlocks) < 0.01) {
    double tmpBurstRate;
    /* ------------------ Negotiate up ----------------- */

    if ((blockSize[0] * 2) <= 1448) {
      blockSize[0] *= 2;
      tRONBlockSize = blockSize[0];

      if (((tmpBurstRate = ((blockSize[0]+72)*burstSize[0]*8/baudRate)) < 0.3) || (burstRate[0] == 0)) {
	/* If burst rate calculates to less than 0.3 s OR burst rate has previosly
	 * been 0 s then there should be a FAST connection and maintain burst rate at 0 s */
	burstRate[0] = 0;
      } else {
	burstRate[0] = (unsigned short)tmpBurstRate + 1;	/* increase burstRate */
      }

      tRONBurstRate = (int) burstRate[0];

      return 1; /* blockSize has been doubled */
    }

    burstSize[0]++;
    tRONBurstSize = burstSize[0];

    if (((tmpBurstRate = ((blockSize[0]+72)*burstSize[0]*8/baudRate)) < 0.3) || (burstRate[0] == 0)) {
      /* If burst rate calculates to less than 0.3 s OR burst rate has previosly
       * been 0 s then there should be a FAST connection and maintain burst rate at 0 s */
      burstRate[0] = 0;
    } else {
      burstRate[0] = (unsigned short)tmpBurstRate;	/* increase burstRate */
    }

    tRONBurstRate = (int) burstRate[0];

    return 1;	/* burstRate has been increased by one */
  }     

  /* Neither up nor down */

  return 0;			/* No change */
}

/*----------------------------------------------------------------------
 * TRONResend
 *
 *	Return list of packets to resend. Called before sending CONTROL_RESEND.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	Updates the decision statistics.
 *----------------------------------------------------------------------
 */

TRONList *TRONResend()
{
  tRONSentPackets += tRONResendList.size;
  return (TRONList *)&tRONResendList;
}

/*       E N D   O F   L I N E      */
